"""Modified for NU-Net
"""

# from collections import namedtuple

import torch
from torchvision import models
from nunet.utils import gram_matrix


class Vgg19(torch.nn.Module):
    def __init__(self, requires_grad=False):
        super(Vgg19, self).__init__()
        vgg_pretrained_features = models.vgg19(pretrained=True).features

        self.slice1 = torch.nn.Sequential()
        self.slice2 = torch.nn.Sequential()
        self.slice3 = torch.nn.Sequential()
        self.slice4 = torch.nn.Sequential()
        for x in range(9):
            self.slice1.add_module(str(x), vgg_pretrained_features[x])
        for x in range(9, 18):
            self.slice2.add_module(str(x), vgg_pretrained_features[x])
        for x in range(18, 27):
            self.slice3.add_module(str(x), vgg_pretrained_features[x])
        for x in range(27, 36):
            self.slice4.add_module(str(x), vgg_pretrained_features[x])
        if not requires_grad:
            for param in self.parameters():
                param.requires_grad = False

    def forward(self, X, mode):
        """
        Argumenst:
            mode ({'feature','embedding'})
        """
        h = self.slice1(X)
        h_relu2_2 = h
        h = self.slice2(h)
        h_relu3_4 = h
        h = self.slice3(h)
        h_relu4_4 = h
        h = self.slice4(h)
        h_relu5_4 = h
        # vgg_outputs = namedtuple("VggOutputs", ['relu1_2', 'relu2_2', 'relu3_3', 'relu4_3'])
        # out = vgg_outputs(h_relu2_2, h_relu3_4, h_relu4_4, h_relu5_4)
        # return out

        if mode == 'feature':
            return h_relu2_2, h_relu3_4, h_relu4_4, h_relu5_4

        elif mode == 'embedding':
            feature_x = [h_relu2_2, h_relu3_4, h_relu4_4, h_relu5_4]
            feature_gx = [gram_matrix(f) for f in feature_x]
            return feature_x, feature_gx
        else:
            raise ValueError
